
/* 
If a designer wants to come make this look more pretty, you're more than welcome!
Feel free to submit a PR!
*/

#mychart {
    max-width: 1280px;
    max-height: 800px;
    margin: 0 auto;
    min-height: 400px;
}

#mytable {
    max-width: 1280px;
    max-height: 800px;
    margin: 0 auto;
    border: 2px solid black;
}

#mytable thead {
    font-size: 1.7rem;
    font-weight: bold;
    background-color: #f2f2f2;
    padding: 0.5rem;
}

#mytable th {
    padding: 0.5rem;
}

#mytable td {
    font-size: 1.2rem;
    padding: 0.3rem;
    border-bottom: 1px solid black;
    margin: 0;
}

.kwh input {
    width: 75px;
    height: 24px;
    font-size: 20px;
}
.kwh::before {
    content: '$';
}
.kwh::after {
    content: '/kWh';
}
